package ch.std.genericdto.dto;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.std.genericdto.tools.StringUtil;

public class BaseDTO<T> {

	private static final Logger logger = LoggerFactory.getLogger(BaseDTO.class);

	private Map<String, Object> header;
	private T body;

	public BaseDTO() {
		this.header = new HashMap<String, Object>();
	}

	public  Map<String, Object> getHeader() {
		return header;
	}

	public void setHeader( Map<String, Object> header) {
		this.header = header;
	}

	public T getBody() {
		return body;
	}

	public void setBody(T body) {
		this.body = body;
	}

	@JsonIgnore
    public Object getHeadValue(String key) {
    	return this.header.get(key);
    }

	@JsonIgnore
    public void setHeadValue(String key, Object value) {
    	this.header.put(key,  value);
    }

	@JsonIgnore
	public String getStatus() {
		try {
			return this.header.get("status").toString();
		} catch (Exception e) {
			return null;
		}
	}

	@JsonIgnore
	public String getStatusMessage() {
		try {
			return this.header.get("statusMessage").toString();
		} catch (Exception e) {
			return null;
		}
	}

	@JsonIgnore
	public boolean isSuccess() {
		return "success".equals(this.getStatus());
	}

	@JsonIgnore
	public void setSuccess(String message) {
		try {
			this.header.put("status", "success");
			this.header.put("statusMessage", message);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}

	@JsonIgnore
	public void setStatusSuccessIfNotSet(String message) {
		if (this.header.get("status") != null) {
			return;
		}
		this.setSuccess(message);
	}

	@JsonIgnore
	public boolean isFailure() {
		return "failure".equals(this.getStatus());
	}

	@JsonIgnore
	public void setFailure(String message) {
		try {
			this.header.put("status", "failure");
			this.header.put("statusMessage", message);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}

	@JsonIgnore
	public void setFailure(Throwable t) {
		try {
			this.header.put("status", "failure");
			String message = t.getMessage();
			if (StringUtil.isNullOrEmpty(message)) {
				message = t.toString();
			}
			this.header.put("statusMessage", message);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}
}
